'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SyncOperation = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _request2 = require('./requests/request');

var _errors = require('./errors');

var _cache = require('./requests/cache');

var _cache2 = _interopRequireDefault(_cache);

var _network = require('./requests/network');

var _client = require('./client');

var _client2 = _interopRequireDefault(_client);

var _query = require('./query');

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _orderBy = require('lodash/orderBy');

var _orderBy2 = _interopRequireDefault(_orderBy);

var _sortedUniqBy = require('lodash/sortedUniqBy');

var _sortedUniqBy2 = _interopRequireDefault(_sortedUniqBy);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var appdataNamespace = process.env.KINVEY_DATASTORE_NAMESPACE || 'appdata';
var syncCollectionName = process.env.KINVEY_SYNC_COLLECTION_NAME || 'kinvey_sync';
var idAttribute = process.env.KINVEY_ID_ATTRIBUTE || '_id';

/**
 * Enum for Sync Operations.
 */
var SyncOperation = {
  Create: _request2.RequestMethod.POST,
  Update: _request2.RequestMethod.PUT,
  Delete: _request2.RequestMethod.DELETE
};
Object.freeze(SyncOperation);
exports.SyncOperation = SyncOperation;

var Sync = function () {
  function Sync() {
    _classCallCheck(this, Sync);

    /**
     * @private
     * @type {Client}
     */
    this.client = _client2.default.sharedInstance();
  }

  /**
   * Pathname used to send sync requests.
   *
   * @return {String} sync pathname
   */


  _createClass(Sync, [{
    key: 'count',


    /**
     * Count the number of entities that are waiting to be synced. A query can be
     * provided to only count a subset of entities.
     *
     * @param   {Query}         [query]                     Query
     * @param   {Object}        [options={}]                Options
     * @param   {Number}        [options.timeout]           Timeout for the request.
     * @return  {Promise}                                   Promise
     *
     * @example
     * var sync = new Sync();
     * var promise = sync.count().then(function(count) {
     *   ...
     * }).catch(function(error) {
     *   ...
     * });
     */
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(query) {
        var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
        var syncEntities, request;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                syncEntities = [];

                // Get all sync entities

                request = new _cache2.default({
                  method: _request2.RequestMethod.GET,
                  url: _url2.default.format({
                    protocol: this.client.protocol,
                    host: this.client.host,
                    pathname: this.pathname
                  }),
                  properties: options.properties,
                  query: query,
                  timeout: options.timeout,
                  client: this.client
                });
                _context.next = 4;
                return request.execute().then(function (response) {
                  return response.data;
                });

              case 4:
                syncEntities = _context.sent;


                // Filter the sync entities so that we only perform
                // one sync operation per unique entity.
                syncEntities = (0, _orderBy2.default)(syncEntities, 'key', ['desc']);
                syncEntities = (0, _sortedUniqBy2.default)(syncEntities, function (syncEntity) {
                  return syncEntity.entity[idAttribute];
                });

                // Return the length of sync entities
                return _context.abrupt('return', syncEntities.length);

              case 8:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function count(_x, _x2) {
        return ref.apply(this, arguments);
      }

      return count;
    }()
  }, {
    key: 'addCreateOperation',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee2(collection, entities) {
        var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                return _context2.abrupt('return', this.addOperation(SyncOperation.Create, collection, entities, options));

              case 1:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function addCreateOperation(_x4, _x5, _x6) {
        return ref.apply(this, arguments);
      }

      return addCreateOperation;
    }()
  }, {
    key: 'addUpdateOperation',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee3(collection, entities) {
        var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                return _context3.abrupt('return', this.addOperation(SyncOperation.Update, collection, entities, options));

              case 1:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function addUpdateOperation(_x8, _x9, _x10) {
        return ref.apply(this, arguments);
      }

      return addUpdateOperation;
    }()
  }, {
    key: 'addDeleteOperation',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee4(collection, entities) {
        var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                return _context4.abrupt('return', this.addOperation(SyncOperation.Delete, collection, entities, options));

              case 1:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function addDeleteOperation(_x12, _x13, _x14) {
        return ref.apply(this, arguments);
      }

      return addDeleteOperation;
    }()
  }, {
    key: 'addOperation',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee6() {
        var operation = arguments.length <= 0 || arguments[0] === undefined ? SyncOperation.Create : arguments[0];
        var collection = arguments[1];

        var _this = this;

        var entities = arguments[2];
        var options = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];
        var singular;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                singular = false;

                // Check that a name was provided

                if (collection) {
                  _context6.next = 3;
                  break;
                }

                throw new _errors.SyncError('A name for a collection must be provided to add entities to the sync table.');

              case 3:

                // Cast the entities to an array
                if (!(0, _isArray2.default)(entities)) {
                  singular = true;
                  entities = [entities];
                }

                // Process the array of entities
                _context6.next = 6;
                return Promise.all((0, _map2.default)(entities, function () {
                  var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee5(entity) {
                    var id, query, findConfig, findRequest, response, syncEntities, syncEntity, request;
                    return regeneratorRuntime.wrap(function _callee5$(_context5) {
                      while (1) {
                        switch (_context5.prev = _context5.next) {
                          case 0:
                            if (entity) {
                              _context5.next = 2;
                              break;
                            }

                            return _context5.abrupt('return', null);

                          case 2:

                            // Validate that the entity has an id
                            id = entity[idAttribute];

                            if (id) {
                              _context5.next = 5;
                              break;
                            }

                            throw new _errors.SyncError('An entity is missing an _id. All entities must have an _id in order to be ' + 'added to the sync table.', entity);

                          case 5:

                            // Find an existing sync operation for the entity
                            query = new _query.Query().equalTo('entity._id', id);
                            findConfig = new _request2.KinveyRequestConfig({
                              method: _request2.RequestMethod.GET,
                              url: _url2.default.format({
                                protocol: _this.client.protocol,
                                host: _this.client.host,
                                pathname: _this.pathname
                              }),
                              properties: options.properties,
                              query: query,
                              timeout: options.timeout
                            });
                            findRequest = new _cache2.default(findConfig);
                            _context5.next = 10;
                            return findRequest.execute();

                          case 10:
                            response = _context5.sent;
                            syncEntities = response.data;
                            syncEntity = syncEntities.length === 1 ? syncEntities[0] : { collection: collection, state: {} };

                            // Update the state

                            syncEntity.state = syncEntity.state || {};
                            syncEntity.state.method = operation;

                            // Update the entity
                            syncEntity.entity = entity;

                            // Send a request to save the sync entity
                            request = new _cache2.default({
                              method: _request2.RequestMethod.PUT,
                              url: _url2.default.format({
                                protocol: _this.client.protocol,
                                host: _this.client.host,
                                pathname: _this.pathname
                              }),
                              properties: options.properties,
                              body: syncEntity,
                              timeout: options.timeout
                            });
                            return _context5.abrupt('return', request.execute());

                          case 18:
                          case 'end':
                            return _context5.stop();
                        }
                      }
                    }, _callee5, _this);
                  }));

                  return function (_x22) {
                    return ref.apply(this, arguments);
                  };
                }()));

              case 6:
                return _context6.abrupt('return', singular ? entities[0] : entities);

              case 7:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function addOperation(_x16, _x17, _x18, _x19) {
        return ref.apply(this, arguments);
      }

      return addOperation;
    }()

    /*
     * Sync entities with the network. A query can be provided to
     * sync a subset of entities.
     *
     * @param   {Query}         [query]                     Query
     * @param   {Object}        [options={}]                Options
     * @param   {Number}        [options.timeout]           Timeout for the request.
     * @return  {Promise}                                   Promise
     *
     * @example
     * var sync = new Sync();
     * var promise = sync.push().then(function(response) {
     *   ...
     * }).catch(function(error) {
     *   ...
     * });
     */

  }, {
    key: 'push',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee12(query) {
        var _this2 = this;

        var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

        var batchSize, i, getRequest, response, syncEntities, _ret;

        return regeneratorRuntime.wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                batchSize = 100;
                i = 0;

                // Make a request for the pending sync entities

                getRequest = new _cache2.default({
                  method: _request2.RequestMethod.GET,
                  url: _url2.default.format({
                    protocol: this.client.protocol,
                    host: this.client.host,
                    pathname: this.pathname
                  }),
                  properties: options.properties,
                  query: query,
                  timeout: options.timeout
                });
                _context12.next = 5;
                return getRequest.execute();

              case 5:
                response = _context12.sent;
                syncEntities = response.data;

                if (!(syncEntities.length > 0)) {
                  _context12.next = 11;
                  break;
                }

                _ret = function () {
                  // Filter the sync entities so that we only perform
                  // one sync operation per unique entity.
                  syncEntities = (0, _orderBy2.default)(syncEntities, 'key', ['desc']);
                  syncEntities = (0, _sortedUniqBy2.default)(syncEntities, function (syncEntity) {
                    return syncEntity.entity[idAttribute];
                  });

                  // Sync the entities in batches to prevent exhausting
                  // available network connections
                  var batchSync = function () {
                    var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee11(syncResults) {
                      var promise;
                      return regeneratorRuntime.wrap(function _callee11$(_context11) {
                        while (1) {
                          switch (_context11.prev = _context11.next) {
                            case 0:
                              promise = new Promise(function () {
                                var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee10(resolve) {
                                  var batch, results;
                                  return regeneratorRuntime.wrap(function _callee10$(_context10) {
                                    while (1) {
                                      switch (_context10.prev = _context10.next) {
                                        case 0:
                                          batch = syncEntities.slice(i, i + batchSize);

                                          i += batchSize;

                                          // Get the results of syncing all of the entities
                                          _context10.next = 4;
                                          return Promise.all((0, _map2.default)(batch, function (syncEntity) {
                                            var collection = syncEntity.collection;
                                            var entity = syncEntity.entity;
                                            var originalId = entity[idAttribute];
                                            var method = syncEntity.state.method;

                                            if (method === _request2.RequestMethod.DELETE) {
                                              // Remove the entity from the network.
                                              var request = new _network.NetworkRequest({
                                                method: _request2.RequestMethod.DELETE,
                                                authType: _request2.AuthType.Default,
                                                url: _url2.default.format({
                                                  protocol: _this2.client.protocol,
                                                  host: _this2.client.host,
                                                  pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection + '/' + originalId
                                                }),
                                                properties: options.properties,
                                                timeout: options.timeout,
                                                client: _this2.client
                                              });
                                              return request.execute().then(function () {
                                                // Remove the sync entity
                                                var config = new _request2.KinveyRequestConfig({
                                                  method: _request2.RequestMethod.DELETE,
                                                  url: _url2.default.format({
                                                    protocol: _this2.client.protocol,
                                                    host: _this2.client.host,
                                                    pathname: _this2.pathname + '/' + syncEntity[idAttribute]
                                                  }),
                                                  properties: options.properties,
                                                  timeout: options.timeout
                                                });
                                                var request = new _cache2.default(config);
                                                return request.execute();
                                              }).then(function () {
                                                // Return the result
                                                var result = { _id: originalId, entity: entity };
                                                return result;
                                              }).catch(function () {
                                                var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee7(error) {
                                                  var getNetworkRequest, originalEntity, putCacheRequest;
                                                  return regeneratorRuntime.wrap(function _callee7$(_context7) {
                                                    while (1) {
                                                      switch (_context7.prev = _context7.next) {
                                                        case 0:
                                                          if (!(error instanceof _errors.InsufficientCredentialsError)) {
                                                            _context7.next = 13;
                                                            break;
                                                          }

                                                          _context7.prev = 1;

                                                          // Try and reset the state of the entity
                                                          getNetworkRequest = new _network.NetworkRequest({
                                                            method: _request2.RequestMethod.GET,
                                                            authType: _request2.AuthType.Default,
                                                            url: _url2.default.format({
                                                              protocol: _this2.client.protocol,
                                                              host: _this2.client.host,
                                                              pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection + '/' + originalId
                                                            }),
                                                            properties: options.properties,
                                                            timeout: options.timeout,
                                                            client: _this2.client
                                                          });
                                                          _context7.next = 5;
                                                          return getNetworkRequest.execute().then(function (response) {
                                                            return response.data;
                                                          });

                                                        case 5:
                                                          originalEntity = _context7.sent;
                                                          putCacheRequest = new _cache2.default({
                                                            method: _request2.RequestMethod.PUT,
                                                            url: _url2.default.format({
                                                              protocol: _this2.client.protocol,
                                                              host: _this2.client.host,
                                                              pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection + '/' + originalId
                                                            }),
                                                            properties: options.properties,
                                                            timeout: options.timeout,
                                                            body: originalEntity
                                                          });
                                                          _context7.next = 9;
                                                          return putCacheRequest.execute();

                                                        case 9:
                                                          _context7.next = 13;
                                                          break;

                                                        case 11:
                                                          _context7.prev = 11;
                                                          _context7.t0 = _context7['catch'](1);

                                                        case 13:
                                                          return _context7.abrupt('return', {
                                                            _id: originalId,
                                                            entity: entity,
                                                            error: error
                                                          });

                                                        case 14:
                                                        case 'end':
                                                          return _context7.stop();
                                                      }
                                                    }
                                                  }, _callee7, _this2, [[1, 11]]);
                                                }));

                                                return function (_x28) {
                                                  return ref.apply(this, arguments);
                                                };
                                              }());
                                            } else if (method === _request2.RequestMethod.POST || method === _request2.RequestMethod.PUT) {
                                              // Save the entity to the network.
                                              var _request = new _network.NetworkRequest({
                                                method: method,
                                                authType: _request2.AuthType.Default,
                                                url: _url2.default.format({
                                                  protocol: _this2.client.protocol,
                                                  host: _this2.client.host,
                                                  pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection + '/' + originalId
                                                }),
                                                properties: options.properties,
                                                timeout: options.timeout,
                                                body: entity,
                                                client: _this2.client
                                              });

                                              // If the entity was created locally then delete the autogenerated _id,
                                              // send a POST request, and update the url.
                                              if (method === _request2.RequestMethod.POST) {
                                                delete entity[idAttribute];
                                                _request.method = _request2.RequestMethod.POST;
                                                _request.url = _url2.default.format({
                                                  protocol: _this2.client.protocol,
                                                  host: _this2.client.host,
                                                  pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection
                                                });
                                                _request.body = entity;
                                              }

                                              return _request.execute().then(function (response) {
                                                return response.data;
                                              }).then(function () {
                                                var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee8(entity) {
                                                  var deleteConfig, deleteRequest, putCacheRequest, deleteCacheRequest;
                                                  return regeneratorRuntime.wrap(function _callee8$(_context8) {
                                                    while (1) {
                                                      switch (_context8.prev = _context8.next) {
                                                        case 0:
                                                          // Remove the sync entity
                                                          deleteConfig = new _request2.KinveyRequestConfig({
                                                            method: _request2.RequestMethod.DELETE,
                                                            url: _url2.default.format({
                                                              protocol: _this2.client.protocol,
                                                              host: _this2.client.host,
                                                              pathname: _this2.pathname + '/' + syncEntity[idAttribute]
                                                            }),
                                                            properties: options.properties,
                                                            timeout: options.timeout
                                                          });
                                                          deleteRequest = new _cache2.default(deleteConfig);
                                                          _context8.next = 4;
                                                          return deleteRequest.execute();

                                                        case 4:

                                                          // Save the result of the network request locally.
                                                          putCacheRequest = new _cache2.default({
                                                            method: _request2.RequestMethod.PUT,
                                                            url: _url2.default.format({
                                                              protocol: _this2.client.protocol,
                                                              host: _this2.client.host,
                                                              pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection + '/' + entity[idAttribute]
                                                            }),
                                                            properties: options.properties,
                                                            timeout: options.timeout,
                                                            body: entity
                                                          });
                                                          _context8.next = 7;
                                                          return putCacheRequest.execute().then(function (response) {
                                                            return response.data;
                                                          });

                                                        case 7:
                                                          entity = _context8.sent;

                                                          if (!(method === _request2.RequestMethod.POST)) {
                                                            _context8.next = 12;
                                                            break;
                                                          }

                                                          deleteCacheRequest = new _cache2.default({
                                                            method: _request2.RequestMethod.DELETE,
                                                            url: _url2.default.format({
                                                              protocol: _this2.client.protocol,
                                                              host: _this2.client.host,
                                                              pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection + '/' + originalId
                                                            }),
                                                            properties: options.properties,
                                                            timeout: options.timeout
                                                          });
                                                          _context8.next = 12;
                                                          return deleteCacheRequest.execute();

                                                        case 12:
                                                          return _context8.abrupt('return', {
                                                            _id: originalId,
                                                            entity: entity
                                                          });

                                                        case 13:
                                                        case 'end':
                                                          return _context8.stop();
                                                      }
                                                    }
                                                  }, _callee8, _this2);
                                                }));

                                                return function (_x29) {
                                                  return ref.apply(this, arguments);
                                                };
                                              }()).catch(function () {
                                                var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee9(error) {
                                                  var getNetworkRequest, originalEntity, putCacheRequest;
                                                  return regeneratorRuntime.wrap(function _callee9$(_context9) {
                                                    while (1) {
                                                      switch (_context9.prev = _context9.next) {
                                                        case 0:
                                                          if (!(error instanceof _errors.InsufficientCredentialsError)) {
                                                            _context9.next = 14;
                                                            break;
                                                          }

                                                          _context9.prev = 1;

                                                          if (!(method !== _request2.RequestMethod.POST)) {
                                                            _context9.next = 10;
                                                            break;
                                                          }

                                                          getNetworkRequest = new _network.NetworkRequest({
                                                            method: _request2.RequestMethod.GET,
                                                            authType: _request2.AuthType.Default,
                                                            url: _url2.default.format({
                                                              protocol: _this2.client.protocol,
                                                              host: _this2.client.host,
                                                              pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection + '/' + originalId
                                                            }),
                                                            properties: options.properties,
                                                            timeout: options.timeout,
                                                            client: _this2.client
                                                          });
                                                          _context9.next = 6;
                                                          return getNetworkRequest.execute().then(function (response) {
                                                            return response.data;
                                                          });

                                                        case 6:
                                                          originalEntity = _context9.sent;
                                                          putCacheRequest = new _cache2.default({
                                                            method: _request2.RequestMethod.PUT,
                                                            url: _url2.default.format({
                                                              protocol: _this2.client.protocol,
                                                              host: _this2.client.host,
                                                              pathname: '/' + appdataNamespace + '/' + _this2.client.appKey + '/' + collection + '/' + originalId
                                                            }),
                                                            properties: options.properties,
                                                            timeout: options.timeout,
                                                            body: originalEntity
                                                          });
                                                          _context9.next = 10;
                                                          return putCacheRequest.execute();

                                                        case 10:
                                                          _context9.next = 14;
                                                          break;

                                                        case 12:
                                                          _context9.prev = 12;
                                                          _context9.t0 = _context9['catch'](1);

                                                        case 14:
                                                          return _context9.abrupt('return', {
                                                            _id: originalId,
                                                            entity: entity,
                                                            error: error
                                                          });

                                                        case 15:
                                                        case 'end':
                                                          return _context9.stop();
                                                      }
                                                    }
                                                  }, _callee9, _this2, [[1, 12]]);
                                                }));

                                                return function (_x30) {
                                                  return ref.apply(this, arguments);
                                                };
                                              }());
                                            }

                                            return {
                                              _id: originalId,
                                              entity: entity,
                                              error: new _errors.SyncError('Unable to sync the entity since the method was not recognized.', syncEntity)
                                            };
                                          }));

                                        case 4:
                                          results = _context10.sent;


                                          // Concat the results
                                          syncResults = syncResults.concat(results);

                                          // Sync the remaining entities

                                          if (!(i < syncEntities.length)) {
                                            _context10.next = 8;
                                            break;
                                          }

                                          return _context10.abrupt('return', resolve(batchSync(syncResults)));

                                        case 8:
                                          return _context10.abrupt('return', resolve(syncResults));

                                        case 9:
                                        case 'end':
                                          return _context10.stop();
                                      }
                                    }
                                  }, _callee10, _this2);
                                }));

                                return function (_x27) {
                                  return ref.apply(this, arguments);
                                };
                              }());
                              return _context11.abrupt('return', promise);

                            case 2:
                            case 'end':
                              return _context11.stop();
                          }
                        }
                      }, _callee11, _this2);
                    }));

                    return function batchSync(_x26) {
                      return ref.apply(this, arguments);
                    };
                  }();

                  // Return the result
                  return {
                    v: batchSync([])
                  };
                }();

                if (!((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object")) {
                  _context12.next = 11;
                  break;
                }

                return _context12.abrupt('return', _ret.v);

              case 11:
                return _context12.abrupt('return', []);

              case 12:
              case 'end':
                return _context12.stop();
            }
          }
        }, _callee12, this);
      }));

      function push(_x23, _x24) {
        return ref.apply(this, arguments);
      }

      return push;
    }()

    /**
     * Clear the sync table. A query can be provided to
     * only clear a subet of the sync table.
     *
     * @param   {Query}         [query]                     Query
     * @param   {Object}        [options={}]                Options
     * @param   {Number}        [options.timeout]           Timeout for the request.
     * @return  {Promise}                                   Promise
     *
     * @example
     * var sync = new Sync();
     * var promise = sync.clear().then(function(response) {
     *   ...
     * }).catch(function(error) {
     *   ...
     * });
     */

  }, {
    key: 'clear',
    value: function clear(query) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var request = new _cache2.default({
        method: _request2.RequestMethod.DELETE,
        url: _url2.default.format({
          protocol: this.client.protocol,
          host: this.client.host,
          pathname: this.pathname
        }),
        properties: options.properties,
        query: query,
        timeout: options.timeout
      });
      return request.execute();
    }
  }, {
    key: 'pathname',
    get: function get() {
      return '/' + appdataNamespace + '/' + this.client.appKey + '/' + syncCollectionName;
    }
  }]);

  return Sync;
}();

exports.default = Sync;